import __main__
import impmagic


@impmagic.loader(
    {'module':'os', 'submodule':['mkdir']},
    {'module':'sys', 'submodule':['exit']},
    {'module':'app.display', 'submodule':['logs']}
)
def create_folder(name):
    try:
        mkdir(name)
    except FileExistsError:
        logs(f"Le répertoire {name} existe déjà", "error")
        exit()
    except PermissionError:
        logs(f"Vous n'avez pas la permission de créer le répertoire {{name}}.", "error")
        exit()
    except FileNotFoundError:
        logs(f"Le répertoire parent spécifié n'existe pas ou vous n'avez pas accès au répertoire {name}.", "error")
        exit()

@impmagic.loader(
    {'module':'sys', 'submodule':['exit']},
    {'module':'app.display', 'submodule':['logs']},
    {'module':'os.path', 'submodule':['exists']}
)
def create_file(filename, content=''):
    if exists(filename):
        raise FileExistsError(f"Le fichier {filename} existe déjà.")
        exit()
    try:
        with open(filename, 'w') as f:
            f.write(content)
    except FileNotFoundError:
        logs(f"Le répertoire de destination du fichier {filename} n'existe pas.", "error")
        exit()
    except PermissionError:
        logs(f"Vous n'avez pas la permission d'écrire le fichier {filename}.", "error")
        exit()


@impmagic.loader(
    {'module':'template.toml_format', 'submodule':['DEFAULT_TOML']},
    {'module':'toml_nxs.toml', 'submodule':['TOML'], 'as':'TOMLnxs'},
    {'module':'os.path', 'submodule':['exists', 'join', 'splitext', 'expanduser', 'abspath']},
    {'module':'env_nxs.env', 'submodule':['create_environment', 'get_py_version']},
    {'module':'package.package', 'submodule':['dependency_to_list']},
    {'module':'template.readme', 'submodule':['README_TEMPLATE']},
    {'module':'sys_nxs.host', 'submodule':['path_rep']},
    {'module': 'template', 'submodule': ['default_conf']},
    {'module': 'repository.repo', 'submodule': ['Repo']},
    {'module': 'os', 'submodule': ['getcwd']},
    {'module':'sys'}
)
def create_project(project, data, argument, action):
    if action=='new':
        tfile = TOMLnxs(join(project,'nexus.toml'))
        tfile.new(data)
        projectfolder = join(project, tfile.get_key(DEFAULT_TOML['projectname']['name'], DEFAULT_TOML['projectname']['section']))
    elif action=='init':
        tfile = TOMLnxs('nexus.toml')
        tfile.new(data)
        projectfolder = tfile.get_key(DEFAULT_TOML['projectname']['name'], DEFAULT_TOML['projectname']['section'])
    
    create_folder(projectfolder)

    mainfile = tfile.get_key(DEFAULT_TOML['mainfile']['name'],DEFAULT_TOML['mainfile']['section'])
    if mainfile!=None:
        if mainfile!="__init__.py":
            if action=='new':
                create_file(join(projectfolder, mainfile))
            name, extension = splitext(mainfile)
            create_file(join(projectfolder, "__init__.py"), f"from .{name} import *")
        else:
            create_file(join(projectfolder, "__init__.py"))

    readmefile = tfile.get_key(DEFAULT_TOML['readme']['name'],DEFAULT_TOML['readme']['section'])
    if action=='new':
        create_file(join(project, readmefile), README_TEMPLATE.format(**{"projectname": tfile.get_key(DEFAULT_TOML['projectname']['name'],DEFAULT_TOML['projectname']['section']),"description": tfile.get_key(DEFAULT_TOML['description']['name'],DEFAULT_TOML['description']['section'])}))
    elif action=='init' and not exists(readmefile):
        create_file(readmefile, README_TEMPLATE.format(**{"projectname": tfile.get_key(DEFAULT_TOML['projectname']['name'],DEFAULT_TOML['projectname']['section']),"description": tfile.get_key(DEFAULT_TOML['description']['name'],DEFAULT_TOML['description']['section'])}))

    if 'license_file' in data and data['license_content']!=None:
        create_file(join(project, data['license_file']), data['license_content'])

    if not argument.noenv and (__main__.nxs.conf.load(val='virtualenvs.create', section='',default=default_conf.virtualenvs_create) or argument.env):
        #Nom du projet                    
        projectname = data['projectname']
        #Nettoyage du dossier env avant création                    
        clear_env = __main__.nxs.conf.load(val='virtualenvs.clear', section='',default=default_conf.virtualenvs_clear)
        if argument.clear:
            clear_env = True

        #Pré-installation des dépendances    
        if not hasattr(__main__.nxs, 'py_version'):
            __main__.nxs.py_version = get_py_version(sys.executable)   
        mdm_env = dependency_to_list(data['dependencies'])

        #Mise à jour de pip                    
        upgradepip_env = __main__.nxs.conf.load(val='virtualenvs.upgradepip', section='',default=default_conf.virtualenvs_upgradepip)
        if argument.upgradepip:
            upgradepip_env = True
        
        symlinks_env = __main__.nxs.conf.load(val='virtualenvs.symlinks', section='',default=default_conf.virtualenvs_symlinks)
        sitepackages_env = __main__.nxs.conf.load(val='virtualenvs.system-site-packages', section='',default=default_conf.virtualenvs_system_site_packages)
        proxy_env = __main__.nxs.conf.load(val='proxy', section='',default=default_conf.proxy)

        folder_env = __main__.nxs.conf.load(val='virtualenvs.foldername', section='',default=default_conf.virtualenvs_foldername)
        if hasattr(argument, 'envpath') and argument.envpath!=None:
            folder_env = argument.envpath

        if (argument.inproject and not argument.incache) or (not argument.inproject and not argument.incache and __main__.nxs.conf.load(val='virtualenvs.in-project', section='',default=default_conf.virtualenvs_in_project)):
            if action=='new':
                virtdir = join(project, folder_env, "default")
            elif action=='init':
                virtdir = join(folder_env, "default")
        else:
            #if os.path.isabs(path):
            cachedir = __main__.nxs.conf.load(val='cache-dir', section='',default=default_conf.cache_dir)
            virtdir = join(expanduser(cachedir), folder_env, projectname, "default")
        
        venv_data = create_environment(virtdir, name=projectname, installmodule=mdm_env, clear=clear_env, upgradepip=upgradepip_env, symlinks=symlinks_env, sitepackages=sitepackages_env, proxy=proxy_env)
        
        for element in venv_data:
            tfile.edit_key(element, venv_data[element].replace(join(abspath('.'), project)+path_rep[0],"").replace("\\","/"), "venv.default")
        tfile.edit_key("default", True, "venv.default")

    if not argument.norepo and (__main__.nxs.conf.load(val='repo.create', section='',default=default_conf.repo_create) or argument.repo):
        if action=='new':
            repo = Repo(project, create=True)
        elif action=='init':
            repo = Repo(getcwd(), create=True)
        if __main__.nxs.conf.load(val='repo.branch_autocreate', section='',default=default_conf.repo_branch_autocreate):
            version = tfile.get_key(DEFAULT_TOML['version']['name'],DEFAULT_TOML['version']['section'])
            repo.switch_branch(version, create=True)


@impmagic.loader(
    {'module':'os.path', 'submodule':['join', 'expanduser']},
    {'module':'env_nxs.env', 'submodule':['create_environment', 'get_py_version']},
    {'module':'package.package', 'submodule':['dependency_to_list']},
    {'module':'sys_nxs.host', 'submodule':['path_rep']},
    {'module': 'template', 'submodule': ['default_conf']},
    {'module':'sys'}
)
def create_project_environment(envname, project, data, tfile, first=False, cache=False, prompt=None, clear=False, nocheck=False):
    #Nom du projet                    
    projectname = data['projectname']
    #Nettoyage du dossier env avant création                    
    clear_env = __main__.nxs.conf.load(val='virtualenvs.clear', section='',default=default_conf.virtualenvs_clear)
    if clear_env:
        clear = True

    #Pré-installation des dépendances    
    if not hasattr(__main__.nxs, 'py_version'):
        __main__.nxs.py_version = get_py_version(sys.executable)

    if 'dependencies' in data and data['dependencies']!=None and len(data['dependencies']): 
        mdm_env = dependency_to_list(data['dependencies'], nocheck=nocheck)
    else:
        mdm_env = []


    if mdm_env is None:
        exit()

    #Mise à jour de pip                    
    upgradepip_env = __main__.nxs.conf.load(val='virtualenvs.upgradepip', section='',default=default_conf.virtualenvs_upgradepip)
    
    symlinks_env = __main__.nxs.conf.load(val='virtualenvs.symlinks', section='',default=default_conf.virtualenvs_symlinks)
    sitepackages_env = __main__.nxs.conf.load(val='virtualenvs.system-site-packages', section='',default=default_conf.virtualenvs_system_site_packages)
    proxy_env = __main__.nxs.conf.load(val='proxy', section='',default=default_conf.proxy)

    folder_env = __main__.nxs.conf.load(val='virtualenvs.foldername', section='',default=default_conf.virtualenvs_foldername)

    cachedir = expanduser(__main__.nxs.conf.load(val='cache-dir', section='',default=default_conf.cache_dir))
    
    if cache==False and __main__.nxs.conf.load(val='virtualenvs.in-project', section='',default=default_conf.virtualenvs_in_project):
        virtdir = join(project, folder_env, envname)
    else:
        virtdir = join(cachedir, folder_env, projectname, envname)

    venv_data = create_environment(virtdir, name=envname, installmodule=mdm_env, clear=clear, upgradepip=upgradepip_env, symlinks=symlinks_env, sitepackages=sitepackages_env, proxy=proxy_env, prompt=prompt)
    
    for element in venv_data:
        if venv_data[element].startswith(cachedir):
            tfile.edit_key(element, venv_data[element].replace("\\","/"), "venv."+envname)
        elif project:
            tfile.edit_key(element, venv_data[element].replace(project+path_rep[0],"").replace("\\","/"), "venv."+envname)
        else:
            tfile.edit_key(element, venv_data[element].replace("\\","/"), "venv."+envname)
    
    #Ajoute default si c'est le premier environnement créé
    if first:
        tfile.edit_key("default", True, "venv."+envname)


@impmagic.loader(
    {'module':'datetime'},
    {'module':'analysis.project', 'submodule': ['get_modules_from_project']},
    {'module':'app.display', 'submodule': ['prompt', 'logs']}, 
    {'module':'template.toml_format', 'submodule': ['DEFAULT_TOML']},
    {'module':'structure.license', 'submodule': ['licenses_availables', 'get_license_content', 'get_license_fullname', 'license_exist', 'detect_license']},
    {'module':'toml_nxs.check.check', 'submodule': ['check_list', 'check_dependency_format']},
    {'module':'os.path', 'submodule': ['exists']}
)
def construct_new_data(nameproject, action):
    data = {}
    data['projectname'] = prompt("Nom du projet", nameproject)
    data['version'] = prompt("Version", DEFAULT_TOML['version']['default_value'])
    data['description'] = prompt("Description", DEFAULT_TOML['description']['default_value'])
    data['mainfile'] = prompt("Fichier principal", "__init__.py")
    if action=='init':
        if not exists(data['mainfile']):
            logs(f"Le fichier {data['mainfile']} n'existe pas", "critical")
            exit()
    data['authors'] = check_list(prompt("authors", DEFAULT_TOML['authors']['default_value']), "authors")
    data['maintainers'] = check_list(prompt("maintainers", DEFAULT_TOML['maintainers']['default_value']), "maintainers")

    if action=='init' and license_exist()!=False:
        license_file = license_exist()
        data['license'] = detect_license(license_file)
        data['license_content'] = None
        data['license_file'] = license_file
    else:
        data['license'] = prompt("license", DEFAULT_TOML['license']['default_value'])
        if licenses_availables(data['license']):
            license_content = get_license_content(data['license'])
            if '[year]' in license_content:
                year = prompt("date license", datetime.date.today().strftime("%Y"))
            if '[fullname]' in license_content:
                if len(data['authors'])>0:
                    author = prompt("author license", data['authors'][0])
                else:
                    author = prompt("author license")
            data['license_content'] = license_content.replace('[year]',year).replace('[fullname]', author)
            data['license_file'] = 'LICENSE'

            data['license'] =   (data['license'])

    if action=='new':
        data['dependencies'] = check_dependency_format(prompt("dependencies", DEFAULT_TOML['dependencies']['default_value']))
    else:
        module_project = get_modules_from_project([data['mainfile']])
        data['dependencies'] = check_dependency_format(module_project['dependency'])
        data['package'] = module_project['externe']+module_project['dependency']
        data['include'] = module_project['interne']

    return data


import impmagic

@impmagic.loader(
    {'module':'app.display', 'submodule': ['logs']}
)
def get_type(file):
    if file.endswith(".whl"):
        return "bdist_wheel"
    elif file.endswith(".tar.gz"):
        return "sdist"
    else:
        logs("Unknown distribution format", "critical")
        exit()

@impmagic.loader(
    {'module':'pkginfo'}
)
def get_metadata(file):
    file_type = get_type(file)

    meta = None
    if file_type=='bdist_wheel':
        pkg_info = pkginfo.Wheel(file)
        meta = pkg_info.__dict__
    elif file_type=='sdist':
        pkg_info = pkginfo.SDist(file)
        meta = pkg_info.__dict__

    return meta

#Uploader adapted from the project Poetry https://github.com/python-poetry/poetry
class Uploader:
    @impmagic.loader(
        {'module':'__main__'},
        {'module': 'template', 'submodule': ['default_conf']}
    )
    def __init__(self, files, url=None):
        self.files = files

        if url==None:
            self.url = __main__.nxs.conf.load(val='publish.url', section='',default=default_conf.publish_url)
        else:
            self.url = url

    @impmagic.loader(
        {'module':'getpass', 'submodule':['getpass']}
    )
    def connect(self):
        self._username = __main__.nxs.conf.load(val='publish.username', section='',default='')
        if self._username=='':
            self._username = input("Username: ")
        self._password = getpass('Password: ')

    @impmagic.loader(
        {'module':'requests_toolbelt', 'submodule':['user_agent']}
    )
    def get_user_agent(self):
        agent: str = user_agent("nexus", "0.5.0")
        return agent

    @impmagic.loader(
        {'module':'requests', 'submodule':['adapters']},
        {'module':'urllib3', 'submodule':['util']}
    )
    def get_adapter(self):
        retry = util.Retry(connect=5, total=10, allowed_methods=["GET"], status_forcelist=[500, 501, 502, 503])
        return adapters.HTTPAdapter(max_retries=retry)

    @impmagic.loader(
        {'module':'requests'}
    )
    def make_session(self):
        session = requests.session()
        auth = self.get_auth()
        if auth is not None:
            session.auth = auth

        session.headers["User-Agent"] = self.get_user_agent()
        for scheme in ("http://", "https://"):
            session.mount(scheme, self.get_adapter())

        return session

    def get_auth(self):
        if self._username is None or self._password is None:
            return None
        return (self._username, self._password)

    def upload(self):
        cert = True
        client_cert = None
        self.session = self.make_session()
        self.session.verify = str(cert) if not isinstance(cert, bool) else cert

        if client_cert:
            self.session.cert = str(client_cert)

        try:
            for file in self.files:
                self._upload_file(file)
        finally:
            self.session.close()

    @impmagic.loader(
        {'module':'hashlib'}
    )
    def post_data(self, file):
        file_type = get_type(file)

        if hasattr(hashlib, "blake2b"):
            blake2_256_hash = hashlib.blake2b(digest_size=256 // 8)

        md5_hash = hashlib.md5()
        sha256_hash = hashlib.sha256()
        with open(file, "rb") as fp:
            for content in iter(lambda: fp.read(8192), b""):
                md5_hash.update(content)
                sha256_hash.update(content)

                if hasattr(hashlib, "blake2b"):
                    blake2_256_hash.update(content)

        md5_digest = md5_hash.hexdigest()
        sha2_digest = sha256_hash.hexdigest()
        blake2_256_digest = None
        if hasattr(hashlib, "blake2b"):
            blake2_256_digest = blake2_256_hash.hexdigest()

        meta = get_metadata(file)

        in_meta = [
            ["name","name"],
            ["version","version"],
            ["metadata_version","metadata_version"],
            ["summary","summary"],
            ["home_page","home_page"],
            ["author","author"],
            ["author","author"],
            ["author_email","author_email"],
            ["maintainer","maintainer"],
            ["maintainer_email","maintainer_email"],
            ["license","license"],
            ["description","description"],
            ["description_content_type","description_content_type"],
            ["keywords","keywords"],
            ["platform","platform"],
            ["classifiers","classifiers"],
            ["download_url","download_url"],
            ["supported_platform","supported_platform"],
            ["provides","provides"],
            ["requires","requires"],
            ["obsoletes","obsoletes"],
            ["project_urls","project_urls"],
            ["provides_dist","provides_dist"],
            ["obsoletes_dist","obsoletes_dist"],
            ["requires_dist","requires_dist"],
            ["requires_external","requires_external"],
            ["requires_python","requires_python"]
        ]

        data = {
            "filetype": file_type,
            "pyversion": "source",
            "comment": None,
            "md5_digest": md5_digest,
            "sha256_digest": sha2_digest,
            "blake2_256_digest": blake2_256_digest
        }
        for m in in_meta:
            if m[1] in meta:
                data[m[0]] = meta[m[1]]

        if "name" in data and "version" in data and "filetype" in data and "pyversion" in data:
            return data
        else:
            print("Donnée manquante")
            exit()

    @impmagic.loader(
        {'module':'os.path', 'submodule':['isfile']},
        {'module':'requests'},
        {'module':'requests.exceptions', 'submodule':['ConnectionError', 'HTTPError']},
        {'module':'requests_toolbelt.multipart', 'submodule':['MultipartEncoder', 'MultipartEncoderMonitor']}
    )
    def _upload_file(self,file):
        if not isfile(file):
            print(f"Archive ({file}) does not exist")
            exit()

        data = self.post_data(file)
        data.update({":action": "file_upload","protocol_version": "1"})

        data_to_send = self._prepare_data(data)

        with open(file, "rb") as fp:
            ####
            if "\\" in file:
                file = file.split("\\")
                file = file[len(file)-1]
            ####
            data_to_send.append(("content", (file, fp, "application/octet-stream")))
            encoder = MultipartEncoder(data_to_send)
            monitor = MultipartEncoderMonitor(encoder)

            resp = None
            try:
                resp = self.session.post(
                    self.url,
                    data=monitor,
                    allow_redirects=False,
                    headers={"Content-Type": monitor.content_type},
                    timeout=15,
                )
                if resp is None or 200 <= resp.status_code < 300:
                    print(f"{file} uploaded")
                elif resp.status_code == 301:
                    print("Redirects are not supported")
                    exit()
                elif self._is_file_exists_error(resp):
                    print(f"File {file} already exists")
                else:
                    resp.raise_for_status()
            except (requests.ConnectionError, requests.HTTPError) as e:
                print(e)
                exit()

    def _prepare_data(self, data):
        data_to_send = []
        for key, value in data.items():
            if not isinstance(value, (list, tuple)):
                data_to_send.append((key, value))
            else:
                for item in value:
                    data_to_send.append((key, item))

        return data_to_send

    def _is_file_exists_error(self, response):
        status = response.status_code
        reason = response.reason.lower()
        text = response.text.lower()
        reason_and_text = reason + text

        return (
            status == 409
            or (status == 400 and "already exist" in reason_and_text)
            or (status == 400 and "updating asset" in reason_and_text)
            or (status == 403 and "overwrite artifact" in reason_and_text)
            or (status == 400 and "already been taken" in reason_and_text)
        )


from cx_Freeze import setup, Executable
import os, sys
import toml
import json
import impmagic
from glob import glob
import re

def replace_by_alias(package_list, alias_file):
    try:
        with open(alias_file) as file:
            alias = load(file)

        for package_name, package_info in package_list.copy().items(): 
            if package_name in alias:
                print(package_name)
                fnd = alias[package]
                if isinstance(fnd, str):
                    package_list[fnd] = package_info
                else:
                    if os.name=="nt" and 'windows' in fnd:
                        package_list[fnd['windows']] = package_info
                    elif os.name!="nt" and 'linux' in fnd:
                        package_list[fnd['linux']] = package_info

                del package_list[package_name]

    finally:
        return package_list


def cat_(content):
    from tempfile import NamedTemporaryFile
    from json import dumps
    import pygments.lexers as lexers
    from pygments.formatters import TerminalTrueColorFormatter
    from pygments import highlight

    BUF_SIZE = 65536

    f = NamedTemporaryFile()
    f.write(dumps(content, indent=4).encode())
    f.seek(0)

    try:
        lex = lexers.get_lexer_by_name('json')
        formatter = TerminalTrueColorFormatter(style="monokai")
        
        while True:
            line = f.readline() 
            if not line:
                break
            else:
                print(highlight(line, lex, formatter),end="")
        f.close()
    except:
        with open(file, 'rb') as f:
            while True:
                data = f.read(BUF_SIZE).decode('utf-8', 'ignore').translate({ord('\u0000'): None})
                if not data:
                    break
                print(data)


def load_key(config, section, entry):
    section_split = section.split(".")

    section = config
    for sec in section_split:
        section = section.get(sec, {})

    entry = section.get(entry, None)

    return entry


#Recherche dans les site-packages si le nom du répertoire est différent
def find_real_package_name(package_name):
    if impmagic.get(package_name):
        return package_name

    else:
        for sys_path in sys.path:
            if os.path.isdir(sys_path):
                for file in glob(f"{sys_path}/*"):
                    matcher = re.match(package_dist, os.path.basename(file))
                    if matcher:
                        if matcher.group("name").lower()==package_name:
                            return matcher.group("name")
    return package_name


#Recherche dans les site-packages si le nom du répertoire est différent
def find_package_name(package_name):
    if impmagic.get(package_name):
        return package_name

    else:
        for sys_path in sys.path:
            if os.path.isdir(sys_path):
                for file in glob(f"{sys_path}/*"):
                    matcher = re.match(package_dist, os.path.basename(file))
                    if matcher:
                        if matcher.group("name").lower()==package_name:
                            top_level_file = os.path.join(file, 'top_level.txt')
                            if os.path.exists(top_level_file):
                                with open(top_level_file) as f:
                                    lines = f.read().splitlines()
                                    return next((line for line in lines if not line.startswith('_')), package_name)
    return package_name


code_folder = sys.argv[2]
destination = sys.argv[3]
mainfile = sys.argv[4]
alias_file = sys.argv[5]
action = sys.argv[6]

#Pour recupérer le nom du package depuis le répertoire dist-info
package_dist = r"(?P<name>^([a-zA-Z0-9_.]+))?\-(?P<version>\d+(\.\d+){0,2})*\.dist-info"


with open(sys.argv[1], "r") as f:
    config = toml.load(f)


#fol = os.path.abspath('.')
fol = os.path.abspath(os.path.split(__file__)[0])
sys.path.remove(fol)
sys.path.append(code_folder)

os.chdir(code_folder)

#################################### SET BUILD #####################################
build_exe_options = {}
options = ["packages","includes","excludes","optimize","no_compress"]
for option in options:
    option_data = load_key(config, 'project.build', option)
    if option_data!=None:
        if isinstance(option_data, list) and len(option_data)==0:
            continue

        if option=="includes":
            option_data = replace_by_alias(option_data, alias_file)
            c_option_data = option_data
            option_data = []
            for line in c_option_data:
                option_data.append(find_package_name(line))

        build_exe_options[option] = option_data

build_exe_options['build_exe'] = destination

#################################### EXECUTABLE ####################################
#For GUI Windows
base = load_key(config, 'project.build', 'GUI')
if base==True and sys.platform == "win32":
    base = "Win32GUI"
else:
    base = None
#base='Console'

copyright = load_key(config, 'project.metadata', 'copyright')
if copyright=="":
    copyright=None

icon = load_key(config, 'project.build', 'icon')
if icon=="":
    icon=None

executables = [(Executable(mainfile, copyright=copyright, base=base, icon=icon))]
####################################################################################

setD = []
infos = ['name', 'version', 'description']
for info in infos:
    key = load_key(config, 'project', info)
    if key!=None:
        setD.append(key)
    else:
        print(f"Erreur avec la clé {info}")
        exit()

# On appelle la fonction setup
if action=="build":
    sys.argv = ['payload.py', 'build']
    #sys.argv[1]='build'
    setup(
        name = setD[0],
        version = setD[1],
        description = setD[2],
        options={"build_exe": build_exe_options},
        executables = executables,
    )
else:
    setup = {'name': setD[0], 'version': setD[1], 'description': setD[2], 'options': build_exe_options, 'executables': {'mainfile': mainfile, 'copyright': copyright, 'base': base, 'icon': icon}}
    cat_(setup)


import impmagic

@impmagic.loader(
    {'module': 'cx_Freeze', 'submodule': ['setup', 'Executable']},
    {'module': 'os', 'submodule': ['chdir']},
    {'module': 'os.path', 'submodule': ['abspath']},
    {'module': 'sys', 'submodule': ['path', 'platform', 'argv']}
)
def compile(tfile, projectfolder, code_folder, destination, mainfile):
    fol = abspath('.')
    path.remove(fol)
    path.append(code_folder)
    
    chdir(code_folder)

    #################################### SET BUILD #####################################
    build_exe_options = {}
    options = ["packages","includes","excludes","optimize","no_compress"]
    for option in options:
        option_data = tfile.get_key(option,'project.build')
        if option_data!=None:
            if isinstance(option_data, list) and len(option_data)==0:
                continue
            build_exe_options[option] = option_data

    build_exe_options['build_exe'] = destination

    #################################### EXECUTABLE ####################################
    #For GUI Windows
    base = tfile.get_key('GUI','project.build')
    if base==True and platform == "win32":
        base = "Win32GUI"
    else:
        base = None
    #base='Console'

    copyright = tfile.get_key('copyright','project.metadata')
    if copyright=="":
        copyright=None

    icon = tfile.get_key('icon','project.build')
    if icon=="":
        icon=None

    executables = [(Executable(mainfile,copyright=copyright,base=base,icon=icon))]
    ####################################################################################

    # On appelle la fonction setup
    argv[1]='build'
    setup(
        name = tfile.doc['project']['name'],
        version = tfile.doc['project']['version'],
        description = tfile.doc['project']['description'],
        options={"build_exe": build_exe_options},
        executables = executables,
    )


import impmagic


@impmagic.loader(
	{'module':'app.display', 'submodule': ['print_nxs']},
)
def print_pattern(index, max, arbo):
	arbo = "".join(arbo)
	if index==max-1:
		print_nxs(f"{arbo}└─", end="")
	else:
		print_nxs(f"{arbo}├─", end="")

	
@impmagic.loader(
	{'module':'app.display', 'submodule': ['print_nxs']},
)
def tree(content, lvl=0, arbo_display=["  "]):
	arbo = []

	for i, rep in content.items():
		if isinstance(rep, dict):
			arbo.append({'name': i, 'type': 'dir', 'content': rep})
		else:
			arbo.append({'name': i, 'type': 'file'})

	arbo = sorted(arbo, key=lambda x: x['name'])
	arbo = sorted(arbo, key=lambda x: x['type'])
	#arbo = sorted(arbo, key=lambda x: x['type'], reverse=True)
	
	for i, element in enumerate(arbo):
		if i==len(content)-1:
			print_nxs(f"{''.join(arbo_display)}└─", nojump=True)
		else:
			print_nxs(f"{''.join(arbo_display)}├─", nojump=True)

		if element['type']=='dir':
			print_nxs(f"{element['name']}", color='magenta')
			if i==len(content)-1:
				new_arbo = arbo_display+['   ']
			else:
				new_arbo = arbo_display+['│  ']

			tree(element['content'], lvl+1, arbo_display=new_arbo)
		else:
			print_nxs(f"{element['name']}", color='yellow')


cache_dir = "~\\AppData\\Local\\Nexus\\Cache"
virtualenvs_create = True
virtualenvs_clear = False
virtualenvs_upgradepip = False
virtualenvs_in_project = True
virtualenvs_symlinks = False
virtualenvs_system_site_packages = False
virtualenvs_foldername = ".venv"
proxy = ""
threading = True
threadmax = 4
#virtualenvs.prompt = "{project_name}"
logs_display = True
logs_full = True
logs_date = True
logs_truecolor = False
project_changelog = True
project_deep_search = 3
publish_url = "https://pypi.org/legacy/"
metadata_source = "https://pypi.org"
repo_create = True
repo_commit = "{version} - {date}"
repo_branch_autocreate = True


import impmagic
import __main__

@impmagic.loader(
	{'module':'json'},
	{'module':'app.display', 'submodule':['logs']},
	{'module':'cache.sqlite', 'submodule':['Cache']},
	{'module': 'template', 'submodule': ['default_conf']},
	{'module': 'sys_nxs.host', 'submodule': ['path_rep']},
	{'module':'os.path', 'submodule': ['join', 'expanduser', 'exists', 'isfile']}
)
def list_cache():
	cachefile = join(expanduser(__main__.nxs.conf.load(val='cache-dir', section='',default=default_conf.cache_dir)).replace(path_rep[1], path_rep[0]), 'repo_cache.db')

	if exists(cachefile) and isfile(cachefile):
		dbcache = Cache(cachefile)
		result = {}

		for lines in dbcache.select_all():
			line = []

			for index in ['name', 'version', 'requires_python', 'requires_dist', 'summary', 'author', 'maintainer', 'license', 'home_page', 'vulnerabilities']:
				try:
					converted = json.loads(getattr(lines, index))
					setattr(lines, index, converted)
				except Exception as err:
					pass

			if lines.name not in result:
				result[str(lines.name)] = [lines.version]
			else:
				result[str(lines.name)].append(lines.version)

		sorted_keys = sorted(result.keys())
		sorted_data = {key: result[key] for key in sorted_keys}

		return sorted_data
	else:
		logs("Cache vide")
		return {}


@impmagic.loader(
	{'module':'json'},
	{'module':'app.display', 'submodule':['logs']},
	{'module':'cache.sqlite', 'submodule':['Cache']},
	{'module': 'template', 'submodule': ['default_conf']},
	{'module': 'sys_nxs.host', 'submodule': ['path_rep']},
	{'module':'os.path', 'submodule': ['join', 'expanduser', 'exists', 'isfile']},
	{'module':'re'},
	{'module':'template', 'submodule':['regex']},
)
def remove_from_cache(package_name):
	cachefile = join(expanduser(__main__.nxs.conf.load(val='cache-dir', section='',default=default_conf.cache_dir)).replace(path_rep[1], path_rep[0]), 'repo_cache.db')

	if exists(cachefile) and isfile(cachefile):
		dbcache = Cache(cachefile)

		package_compiled = re.compile(regex.info_package_regex)
		package_match = package_compiled.search(package_name)

		if package_match!=None:
			package_name = package_match.group('name')
			operator = package_match.group('operator')
			version = package_match.group('version')

			if version!=None:
				if operator:
					dbcache.remove_package(package_name, version, operator)
				else:
					dbcache.remove_package(package_name, version)
			else:
				dbcache.remove_package(package_name)
		else:
			logs("Format de nom invalide", "critical")
	else:
		logs("Cache vide")
		return {}


@impmagic.loader(
	{'module':'re'},
	{'module':'template', 'submodule':['regex']},
	{'module':'app.display', 'submodule': ['logs']},
	{'module':'package.package', 'submodule': ['get_package_information', 'get_versionlist_package', 'compare_version']},
	{'module':'cache.sqlite', 'submodule':['Cache']},
	{'module': 'template', 'submodule': ['default_conf']},
	{'module':'os.path', 'submodule': ['join', 'expanduser']},
	{'module': 'sys_nxs.host', 'submodule': ['path_rep']},
)
def add_to_cache(package_name):
	cachefile = join(expanduser(__main__.nxs.conf.load(val='cache-dir', section='',default=default_conf.cache_dir)).replace(path_rep[1], path_rep[0]), 'repo_cache.db')
	dbcache = Cache(cachefile)

	package_compiled = re.compile(regex.info_package_regex)
	package_match = package_compiled.search(package_name)

	if package_match!=None:
		package_name = package_match.group('name')
		operator = package_match.group('operator')
		version = package_match.group('version')

		versionlist = get_versionlist_package(package_name)

		version_added = []

		#Stocke les versions dans le cache pour éviter d'afficher un message pour les versions déjà en cache
		version_already_in_cache = []
		for line in dbcache.search(package_name):
			version_already_in_cache.append(line['version'])

		
		if version!=None:
			if operator:	
				for vers in versionlist:
					if compare_version(vers, operator, version):
						get_package_information(package_name, vers)
						version_added.append(vers)
			else:
				get_package_information(package_name, version)
				version_added.append(version)
		else:
			for vers in versionlist:
				get_package_information(package_name, vers)
				version_added.append(vers)

		if len(version_added):
			for vers in version_added:
				if vers not in version_already_in_cache:
					result = dbcache.search(package_name, version=vers)

					if result:
						logs(f"Ajout de {package_name}=={vers}")

		else:
			logs("Aucune version trouvée")
	else:
		logs("Format de nom invalide", "critical")


TOML_TEMPLATE= """\
{{
    "project": {{
        "name": "{projectname}",
        "version": "{version}",
        "description": "{description}",
        "mainfile": "{mainfile}",
        "requires-python": "{pythonversion}",
        "metadata": {{
            "authors": {authors},
            "maintainers": {maintainers},
            "readme": "{readme}",
            "changelog": "{changelog}",
            "license": "{license}",
            "license_file": "{license_file}",
            "keywords": {keywords},
            "classifiers": {classifiers},
            "copyright": "{copyright}"
        }},
        "dependencies": {{
            "windows": {dependencies_win},
            "linux": {dependencies_linux},
        }},
        "build": {{
            "packages": {package},
            "includes": {include},
            "excludes": {exclude},
            "include_files": {include},
            "optimize": {optimize},
            "no_compress": {compress},
            "GUI": {gui},
            "icon": "{icon}"
        }},
        "wheel": {{
            "platforms": "{platforms}",
            "entry_points": {{
                "console_scripts": "{console_scripts}"
            }}
        }},
        "urls": {{
            "homepage": "{homepage}",
            "documentation": "{documentation}",
            "changelog": "{changelog}"
        }}
    }}
}}
"""


import impmagic


class Analyser():
    @impmagic.loader(
        {'module':'pylint.lint', 'submodule': ['Run']}
    )
    def __init__(self, filename, exclude=""):
        self.init_reporter()

        Run([f'--disable={exclude}',filename], reporter=self.reporter, exit=False)

    @impmagic.loader(
        {'module':'pylint.reporters', 'submodule': ['JSONReporter']},
        {'module':'io', 'submodule': ['StringIO']}
    )
    def init_reporter(self):
        self.out_stream = StringIO()
        self.reporter = JSONReporter(self.out_stream)

    @impmagic.loader(
        {'module':'json', 'submodule': ['load']}
    )
    def load_report(self):
        self.out_stream.seek(0)
        js = load(self.out_stream)

        self.report = {}
        for rep in js:
            if rep['path'] not in self.report:
                self.report[rep['path']] = {}

            if rep['type'] not in self.report[rep['path']]:
                self.report[rep['path']][rep['type']] = []

            self.report[rep['path']][rep['type']].append(f'{rep["message-id"]: <6} - {rep["line"]}:{rep["column"]} - {rep["message"]}')

    @impmagic.loader(
        {'module':'app.display', 'submodule': ['print_nxs']}
    )
    def display_report(self):
        if hasattr(self, "report"):
            for file in self.report:
                print_nxs(f"\n{file}")
                for type_e in self.report[file]:
                    print(f"  {type_e}")
                    for el in self.report[file][type_e]:
                        print(f"     {el}")

    def get_note(self):
        if hasattr(self, "reporter"):
            return f"{round(self.reporter.linter.stats.global_note, 2)}/10"



import impmagic
import __main__

class BanditScan():
    @impmagic.loader(
        {'module':'bandit.core', 'submodule': ['constants']},
        {'module':'bandit.core', 'submodule': ['config'], 'as': 'b_config'},
        {'module':'bandit.core', 'submodule': ['manager'], 'as': 'b_manager'},
    )
    def __init__(self, file, exclude_file=None, recursive=True, severity=1, confidence=1, show_errors=False, show_metrics= False, show_globalmetrics=False, show_filemetrics=False, show_details=False, show_all=True):
        self.recursive = recursive
        self.show_errors = show_errors
        self.show_metrics = show_metrics
        self.show_globalmetrics = show_globalmetrics
        self.show_filemetrics = show_filemetrics
        self.show_details = show_details
        self.show_all = show_all

        self.sev_level = constants.RANKING[severity - 1]
        self.conf_level = constants.RANKING[confidence - 1]

        b_conf = b_config.BanditConfig(config_file=None)

        profile = {}
        profile["include"] = set(b_conf.get_option("tests") or [])
        profile["exclude"] = set(b_conf.get_option("skips") or [])

        self.b_mgr = b_manager.BanditManager(b_conf,None,False,profile=profile,verbose=False,quiet=False,ignore_nosec=False)

        if exclude_file is None:
            exclude_file = ",".join(constants.EXCLUDE)
        else:
            exclude_file = ",".join(exclude_file)

        self.b_mgr.discover_files(file, self.recursive, exclude_file)

    @impmagic.loader(
        {'module':'zpp_ManagedFile', 'submodule': ['ManagedFile']},
        {'module':'json', 'submodule': ['load']}
    )
    def run_scan(self):
        self.b_mgr.run_tests()

        file = ManagedFile(closable=False)

        self.b_mgr.output_results(3,self.sev_level,self.conf_level,file,'json',None)

        file.seek(0)
        self.result = load(file)

    @impmagic.loader(
        {'module':'zpp_color', 'submodule': ['fg', 'attr']}
    )
    def output(self):
        if self.show_errors or self.show_all:
            errors = self.result['errors']
            if len(errors)==0:
                print(f"\n{fg(__main__.color['yellow'])}Errors:{attr(0)} No error notify")
            else:
                errors = "\n".join(errors)
                print(f'Errors: {errors}')

        metrics = self.result['metrics']
        if self.show_metrics or self.show_globalmetrics or self.show_all:
            print(f"\n{fg(__main__.color['yellow'])}Metrics total:{attr(0)}")
            metric = metrics['_totals']
            for line in metric:
                print(f"  {line}: {metric[line]}")
        if self.show_metrics or self.show_filemetrics or self.show_all:
            print(f"\n{fg(__main__.color['yellow'])}Metrics by file:{attr(0)}")
            for metric in metrics:
                if metric!='_totals':
                    print(f"- {metric}")
                    cont = metrics[metric]
                    for line in cont:
                        print(f"  {str(line)}: {cont[line]}")

        if self.show_details or self.show_all:
            print(f"\n{fg(__main__.color['yellow'])}Details:{attr(0)}",end="")
            details = self.result['results']
            for detail in details:
                print(f"\n {fg(__main__.color['dark_gray'])}filename:{attr(0)} {detail['filename']}")
                print(f" {fg(__main__.color['dark_gray'])}test name:{attr(0)} {detail['test_name']}")
                print(f" {fg(__main__.color['dark_gray'])}confidence lvl:{attr(0)} {detail['issue_confidence']}")
                print(f" {fg(__main__.color['dark_gray'])}severity lvl:{attr(0)} {detail['issue_severity']}")
                print(f" {fg(__main__.color['dark_gray'])}issue:{attr(0)} {detail['issue_text']}")
                print(f" {fg(__main__.color['dark_gray'])}issue cwe:{attr(0)} {detail['issue_cwe'].get('link','')}")
                print(f" {fg(__main__.color['dark_gray'])}line:{attr(0)} {detail['line_number']}")
                print(f" {fg(__main__.color['dark_gray'])}column:{attr(0)} {detail['col_offset']}")
                code = "   "+detail['code'].replace("\n","\n   ")
                print(f" {fg(__main__.color['dark_gray'])}code:{attr(0)} \n{code}")
            print("")